"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.List = void 0;
const Node_1 = require("./Node");
class List {
    constructor() {
        this.length = 0;
        this.head = null;
        this.tail = null;
    }
    isEmpty() {
        return this.length === 0;
    }
    _getNode(index) {
        if (index < 0 || index >= this.length) {
            return null;
        }
        let currentNode;
        if (index < this.length / 2) {
            currentNode = this.head;
            for (let i = 0; i < index; i++) {
                currentNode = currentNode.next;
            }
        }
        else {
            currentNode = this.tail;
            for (let i = this.length - 1; i > index; i--) {
                currentNode = currentNode.prev;
            }
        }
        return currentNode;
    }
    toArray() {
        const array = [];
        let currentNode = this.head;
        while (currentNode !== null) {
            array.push(currentNode.data);
            currentNode = currentNode.next;
        }
        return array;
    }
    push(data) {
        const node = new Node_1.Node(data);
        if (this.isEmpty()) {
            this.head = node;
            this.tail = node;
        }
        else {
            this.tail.next = node;
            node.prev = this.tail;
            this.tail = node;
        }
        this.length++;
    }
    findIndex(data) {
        let currentIndex = 0;
        let currentNode = this.head;
        while (currentNode !== null) {
            if (currentNode.data === data) {
                return currentIndex;
            }
            currentNode = currentNode.next;
            currentIndex++;
        }
        return -1;
    }
    removeByIndex(index) {
        const nodeToRemove = this._getNode(index);
        if (!nodeToRemove) {
            return null;
        }
        return this._removeNode(nodeToRemove);
    }
    getData(index) {
        const node = this._getNode(index);
        return node ? node.data : null;
    }
    removeByData(data) {
        let currentNode = this.head;
        while (currentNode !== null) {
            if (currentNode.data === data) {
                return this._removeNode(currentNode);
            }
            currentNode = currentNode.next;
        }
        return null;
    }
    _removeNode(nodeToRemove) {
        if (nodeToRemove === this.head) {
            this.head = nodeToRemove.next;
        }
        if (nodeToRemove === this.tail) {
            this.tail = nodeToRemove.prev;
        }
        if (nodeToRemove.prev !== null) {
            nodeToRemove.prev.next = nodeToRemove.next;
        }
        if (nodeToRemove.next !== null) {
            nodeToRemove.next.prev = nodeToRemove.prev;
        }
        nodeToRemove.prev = null;
        nodeToRemove.next = null;
        this.length--;
        return nodeToRemove.data;
    }
}
exports.List = List;
//# sourceMappingURL=List.js.map